RwArc

Struct RwArc 

pub struct RwArc<T>(/* private fields */);
A reference-counting pointer with read-write capabilities.

This is essentially Arc<RwLock<T>>, so it can provide read-write capabilities through RwArc::read and RwArc::write.

In addition, this allows to derive another reference-counting pointer with read-only capabilities (RoArc) via RwArc::clone_ro.

The purpose of having this type is to allow lockless (read) access to the underlying data when there is only one RwArc instance for the particular allocation (note that there can be any number of RoArc instances for that allocation). See the [RwArc::get] method for more details.

Implementations

Source

impl<T> RwArc<T>

pub closed spec fn wf(self) -> bool

Source

impl<T> RwArc<T>

pub exec fn new(data: T) -> Self

Creates a new RwArc<T>.

pub exec fn read(&self) -> RwLockReadGuard<'_, T, PreemptDisabled>

Acquires the read lock for immutable access.

pub exec fn write(&self) -> RwLockWriteGuard<'_, T, PreemptDisabled>

Acquires the write lock for mutable access.

pub exec fn clone_ro(&self) -> RoArc<T>

Clones a RoArc that points to the same allocation.

Source

impl<T: Clone> RwArc<T>

pub exec fn get_cloned(&self) -> T

Returns the contained value by cloning it.