pub struct RwMutexWriteGuard<'a, T> { /* private fields */ }Expand description
A guard that provides mutable data access.
Implementations
Sourceimpl<'a, T> RwMutexWriteGuard<'a, T>
impl<'a, T> RwMutexWriteGuard<'a, T>
Sourceimpl<'a, T> RwMutexWriteGuard<'a, T>
impl<'a, T> RwMutexWriteGuard<'a, T>
Sourcepub exec fn downgrade(self) -> RwMutexUpgradeableGuard<'a, T>
pub exec fn downgrade(self) -> RwMutexUpgradeableGuard<'a, T>
Atomically downgrades a write guard to an upgradeable reader guard.
This method always succeeds because the lock is exclusively held by the writer.