WaitQueue

Struct WaitQueue 

pub struct WaitQueue { /* private fields */ }
A wait queue.

One may wait on a wait queue to put its executing thread to sleep. Multiple threads may be the waiters of a wait queue. Other threads may invoke the wake-family methods of a wait queue to wake up one or many waiting threads.

Implementations

Source

impl WaitQueue

pub closed spec fn type_inv(self) -> bool

Source

impl WaitQueue

pub const exec fn new() -> Self

Creates a new, empty wait queue.

pub exec fn wait_until<F, R>(&self, cond: F) -> ret : R
where F: FnMut() -> Option<R>,

requires
cond.requires(()),
ensures
cond.ensures((), Some(ret)),

Waits until some condition is met.

This method takes a closure that tests a user-given condition. The method only returns if the condition returns Some(_). A waker thread should first make the condition Some(_), then invoke the wake-family method. This ordering is important to ensure that waiter threads do not lose any wakeup notifications.

By taking a condition closure, this wait-wakeup mechanism becomes more efficient and robust.

pub exec fn wake_one(&self) -> r : bool

Wakes up one waiting thread, if there is one at the point of time when this method is called, returning whether such a thread was woken up.

pub exec fn wake_all(&self) -> r : usize

Wakes up all waiting threads, returning the number of threads that were woken up.