pub struct Waker { /* private fields */ }Implementations
Sourceimpl Waker
impl Waker
Sourcepub exec fn wake_up(&self) -> bool
pub exec fn wake_up(&self) -> bool
Wakes up the associated Waiter.
This method returns true if the waiter is woken by this call. It returns false if the
waiter has already been woken by a previous call to the method, or if the waiter has been
dropped.
Note that if this method returns true, it implies that the wake event will be properly
delivered, or that the waiter will be dropped after being woken. It’s up to the caller to
handle the latter case properly to avoid missing the wake event.