Once

Type Alias Once 

pub type Once<V, Own, F> = OnceImpl<AtomicDataWithOwner<V, Own>, F>;
A Once that combines some data with a permission to access it.

This type alias automatically lifts the target value V into a wrapper AtomicDataWithOwner<V, Own> where Own is the permission type so that we can reason about non-trivial runtime properties in verification.

Aliased Type

pub struct Once<V, Own, F> { /* private fields */ }