pub enum LocalIrqDisabled {}Expand description
A guardian that disables IRQs while holding a lock.
This guardian would incur a certain time overhead over
PreemptDisabled. So prefer avoiding using this guardian when
IRQ handlers are allowed to get executed while holding the
lock. For example, if a lock is never used in the interrupt
context, then it is ok not to use this guardian in the process context.