pub struct RwMutexUpgradeableGuard<'a, T> { /* private fields */ }Expand description
A guard that provides immutable data access but can be atomically
upgraded to RwMutexWriteGuard.
Implementations
Sourceimpl<'a, T> RwMutexUpgradeableGuard<'a, T>
impl<'a, T> RwMutexUpgradeableGuard<'a, T>
Sourceimpl<'a, T> RwMutexUpgradeableGuard<'a, T>
impl<'a, T> RwMutexUpgradeableGuard<'a, T>
Sourcepub exec fn upgrade(self) -> RwMutexWriteGuard<'a, T>
pub exec fn upgrade(self) -> RwMutexWriteGuard<'a, T>
Upgrades this upread guard to a write guard atomically.
After calling this method, subsequent readers will be blocked while previous readers remain unaffected.
The calling thread will not sleep, but spin to wait for the existing reader to be released. There are two main reasons.
- First, it needs to sleep in an extra waiting queue and needs extra wake-up logic and overhead.
- Second, upgrading method usually requires a high response time (because the mutex is being used now).